#!/bin/bash
#

# buildBoost.sh <opt|debug|prof> [e2|e1|gcc47|gcc46]
# default is gcc47

extraqual=${1}
basequal=${2}

if [ -z ${basequal} ]
then
    basequal=gcc47
fi


vers=v1_49_0
bvers=boost_1_49_0

if [ "${extraqual}" = "opt" ]
then
    extra_command="variant=release"
elif [ "${extraqual}" = "debug" ]
then
    extra_command="variant=debug"
elif [ "${extraqual}" = "prof" ]
then
    extra_command="variant=profile"
else
   echo "ERROR: please define the extra qualifier"
   echo "USAGE: buildBoost.sh <opt|debug|prof>"
   exit 1
fi
fullqual=${basequal}:${extraqual}
qualdir=${basequal}-${extraqual}

if [ "${basequal}" = "e2" ] ||  [ "${basequal}" = "gcc47" ]
then
   gccdotver=4.7
   gccver=v4_7_1
else
   gccdotver=4.6
   gccver=v4_6_1
fi

echo "compiling for ${fullqual}"


source ${PRODUCTS}/setup
OS=`uname`
if [ ${OS} = "Darwin" ]
then
    flvr=`ups flavor -2`
else
    flvr=`ups flavor`
fi
boostdir=${PRODUCTS}/boost/${vers}/${flvr}-${qualdir}
sourcedir=${PRODUCTS}/boost/${vers}/source

if [ -z ${flvr} ]
then
   echo "ERROR: could not determine flavor for this platform"
   echo "       something is very wrong"
   exit 1
fi

# setup gcc
setup gcc ${gccver}
gccshortver=`gcc --version | head -1 | cut -f3 -d" " | cut -f1-2 -d"."`
gccfullver=`gcc --version | head -1 | cut -f3 -d" "`
if [ ${gccshortver} = ${gccdotver} ]
then
  echo " OK - gcc version is ${gccfullver}"
else
  echo "ERROR: wrong gcc version ${gccfullver}"
  exit 1
fi

echo "Building boost in ${boostdir}"

mkdir -p ${boostdir}/build
if [ ! -d ${boostdir}/build ]
then
   echo "ERROR: failed to create ${boostdir}/build"
   exit 1
fi

set -x

cd ${boostdir}/build
tar -xzf ${sourcedir}/${bvers}.tar.gz
# apply patch for gcc 4.7 on Darwin
if [ ${OS} = "Darwin" ]
then
   cd ${boostdir}/build
   patch -b -p0 < ${sourcedir}/${bvers}.Darwin.patch
fi

cd ${boostdir}/build/${bvers}
./bootstrap.sh --prefix=${boostdir} --without-icu --without-libraries=python,mpi

if [ ${OS} = "Darwin" ]
then
  mv project-config.jam project-config.jam.orig
  sed s%GCCDIR%${GCC_FQ_DIR}%g ${sourcedir}/project-config.jam | sed s%BOOSTDIR%${boostdir}%g > ${boostdir}/build/${bvers}/project-config.jam
fi

# --debug-building  provides verbose output
#./bjam --debug-building "${cppcmd}" "${extra_command}"
#./bjam --debug-building  "${extra_command}" install
# 1.49.0: use b2 instead of bjam?
if [ ${OS} = "Darwin" ]
then
  if [ "${basequal}" = "e2" ] ||  [ "${basequal}" = "e1" ]
  then
    ./b2 cxxflags="-std=c++11 -Wno-deprecated-declarations -fvisibility-inlines-hidden" "${extra_command}"
  else
    ./b2 cxxflags="-fvisibility-inlines-hidden" "${extra_command}" -d+2 
  fi
else
  if [ "${basequal}" = "e2" ] ||  [ "${basequal}" = "e1" ]
  then
    ./b2 cxxflags="-std=c++0x -Wno-deprecated-declarations" "${extra_command}"
  else
    ./b2 "${extra_command}"
  fi
fi
./b2 "${extra_command}" install

set +x

if [ ! -d ${boostdir}/lib ]
then
   echo "ERROR: failed to create ${boostdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# this should not complain
echo "Finished building boost ${vers}"
echo "Declaring boost ${vers} ${flvr} ${fullqual}"
if [ ${OS} = "Darwin" ]
then
   ups declare boost ${vers} -r boost/${vers} -2 -m boost.table -q ${fullqual}
else 
   ups declare boost ${vers} -r boost/${vers} -4 -m boost.table -q ${fullqual}
fi
setup boost ${vers} -q ${fullqual}
echo "BOOST_LIB is ${BOOST_LIB}"

exit 0
